@@ -26,7 +26,6 @@ struct reg_file_info {
 extern int open_reg_by_id(u32 id);
 extern int open_path_by_id(u32 id, int (*open_cb)(struct reg_file_info *, void *), void *arg);
 extern void clear_ghost_files(void);
-extern int collect_reg_files(void);
 
 extern int prepare_shared_reg_files(void);
 
@@ -36,4 +35,7 @@ extern int dump_one_reg_file(int lfd, u32 id, const struct fd_parms *p);
 extern struct file_remap *lookup_ghost_remap(u32 dev, u32 ino);
 extern void remap_put(struct file_remap *remap);
 
+extern struct collect_image_info reg_file_cinfo;
+extern struct collect_image_info remap_cinfo;
+
 #endif /* __CR_FILES_REG_H__ */
