@@ -3,10 +3,6 @@
 
 #include "crtools.h"
 
-extern void show_ipc_var(int fd);
-extern void show_ipc_shm(int fd);
-extern void show_ipc_msg(int fd);
-extern void show_ipc_sem(int fd);
 extern int dump_ipc_ns(int ns_pid, const struct cr_fdset *fdset);
 extern int prepare_ipc_ns(int pid);
 
