@@ -42,11 +42,6 @@ struct packet_sock_desc {
 
 #define NO_FANOUT	((unsigned int)-1)
 
-void show_packetsk(int fd)
-{
-	pb_show_plain_pretty(fd, PB_PACKETSK, "5:%d");
-}
-
 static int dump_mreqs(PacketSockEntry *psk, struct packet_sock_desc *sd)
 {
 	int i;
@@ -194,7 +189,7 @@ static int dump_one_packet_fd(int lfd, u32 id, const struct fd_parms *p)
 	if (ret)
 		goto out;
 
-	ret = pb_write_one(fdset_fd(glob_fdset, CR_FD_PACKETSK), &psk, PB_PACKETSK);
+	ret = pb_write_one(fdset_fd(glob_fdset, CR_FD_PACKETSK), &psk, PB_PACKET_SOCK);
 out:
 	release_skopts(&skopts);
 	xfree(psk.rx_ring);
@@ -496,18 +491,13 @@ static int collect_one_packet_sk(void *o, ProtobufCMessage *base)
 	struct packet_sock_info *si = o;
 
 	si->pse = pb_msg(base, PacketSockEntry);
-	file_desc_add(&si->d, si->pse->id, &packet_sock_desc_ops);
-
-	return 0;
+	return file_desc_add(&si->d, si->pse->id, &packet_sock_desc_ops);
 }
 
-int collect_packet_sockets(void)
-{
-	int ret = collect_image(CR_FD_PACKETSK, PB_PACKETSK,
-			sizeof(struct packet_sock_info), collect_one_packet_sk);
-
-	if (ret < 0 && errno == ENOENT)
-		return 0;
-
-	return ret;
-}
+struct collect_image_info packet_sk_cinfo = {
+	.fd_type = CR_FD_PACKETSK,
+	.pb_type = PB_PACKET_SOCK,
+	.priv_size = sizeof(struct packet_sock_info),
+	.collect = collect_one_packet_sk,
+	.flags = COLLECT_OPTIONAL,
+};
