@@ -17,7 +17,7 @@
 #include "image.h"
 #include "log.h"
 #include "util.h"
-#include "util-net.h"
+#include "util-pie.h"
 #include "sockets.h"
 #include "sk-queue.h"
 #include "mount.h"
@@ -197,6 +197,23 @@ static int dump_one_unix_fd(int lfd, u32 id, const struct fd_parms *p)
 	} else if (ue.state == TCP_ESTABLISHED) {
 		const struct unix_sk_listen_icon *e;
 
+		e = lookup_unix_listen_icons(ue.ino);
+		if (!e) {
+			/*
+			 * ESTABLISHED socket without peer and without
+			 * anyone waiting for it should be semi-closed
+			 * connection.
+			 */
+
+			if (ue.shutdown == SK_SHUTDOWN__BOTH) {
+				pr_info("Dumping semi-closed connection\n");
+				goto dump;
+			}
+
+			pr_err("Dangling connection %#x\n", ue.ino);
+			goto err;
+		}
+
 		/*
 		 * If this is in-flight connection we need to figure
 		 * out where to connect it on restore. Thus, tune up peer
@@ -206,12 +223,6 @@ static int dump_one_unix_fd(int lfd, u32 id, const struct fd_parms *p)
 		 * not now, just to reduce size of dump files.
 		 */
 
-		e = lookup_unix_listen_icons(ue.ino);
-		if (!e) {
-			pr_err("Dangling in-flight connection %d\n", ue.ino);
-			goto err;
-		}
-
 		/* e->sk_desc is _never_ NULL */
 		if (e->sk_desc->state != TCP_LISTEN) {
 			pr_err("In-flight connection on "
@@ -224,11 +235,11 @@ static int dump_one_unix_fd(int lfd, u32 id, const struct fd_parms *p)
 		pr_debug("\t\tFixed inflight socket %#x peer %#x)\n",
 				ue.ino, ue.peer);
 	}
-
+dump:
 	if (dump_socket_opts(lfd, &skopts))
 		goto err;
 
-	if (pb_write_one(fdset_fd(glob_fdset, CR_FD_UNIXSK), &ue, PB_UNIXSK))
+	if (pb_write_one(fdset_fd(glob_fdset, CR_FD_UNIXSK), &ue, PB_UNIX_SK))
 		goto err;
 
 	if (sk->rqlen != 0 && !(sk->type == SOCK_STREAM &&
@@ -450,7 +461,7 @@ int fix_external_unix_sockets(void)
 		e.fown		= &fown;
 		e.opts		= &skopts;
 
-		if (pb_write_one(fdset_fd(glob_fdset, CR_FD_UNIXSK), &e, PB_UNIXSK))
+		if (pb_write_one(fdset_fd(glob_fdset, CR_FD_UNIXSK), &e, PB_UNIX_SK))
 			goto err;
 
 		show_one_unix_img("Dumped extern", &e);
@@ -486,11 +497,6 @@ static struct unix_sk_info *find_unix_sk_by_ino(int ino)
 	return NULL;
 }
 
-void show_unixsk(int fd)
-{
-	pb_show_plain_pretty(fd, PB_UNIXSK, "1:%#x 2:%#x 3:%d 4:%d 5:%d 6:%d 7:%d 8:%#x 11:S");
-}
-
 static int shutdown_unix_sk(int sk, struct unix_sk_info *ui)
 {
 	int how;
@@ -710,10 +716,44 @@ static int open_unixsk_standalone(struct unix_sk_info *ui)
 	pr_info("Opening standalone socket (id %#x ino %#x peer %#x)\n",
 			ui->ue->id, ui->ue->ino, ui->ue->peer);
 
-	sk = socket(PF_UNIX, ui->ue->type, 0);
-	if (sk < 0) {
-		pr_perror("Can't make unix socket");
-		return -1;
+	if ((ui->ue->state == TCP_ESTABLISHED) && !ui->ue->peer) {
+		int ret, sks[2];
+
+		if (ui->ue->type != SOCK_STREAM) {
+			pr_err("Non-stream socket %x in established state\n",
+					ui->ue->ino);
+			return -1;
+		}
+
+		if (ui->ue->shutdown != SK_SHUTDOWN__BOTH) {
+			pr_err("Wrong shutdown/peer state for %x\n",
+					ui->ue->ino);
+			return -1;
+		}
+
+		ret = socketpair(PF_UNIX, ui->ue->type, 0, sks);
+		if (ret < 0) {
+			pr_perror("Can't create socketpair");
+			return -1;
+		}
+
+		/*
+		 * Restore queue at the one end,
+		 * before closing the second one.
+		 */
+		if (restore_sk_queue(sks[1], ui->ue->id)) {
+			pr_perror("Can't restore socket queue");
+			return -1;
+		}
+
+		close(sks[1]);
+		sk = sks[0];
+	} else {
+		sk = socket(PF_UNIX, ui->ue->type, 0);
+		if (sk < 0) {
+			pr_perror("Can't make unix socket");
+			return -1;
+		}
 	}
 
 	if (bind_unix_sk(sk, ui))
@@ -783,25 +823,24 @@ static int collect_one_unixsk(void *o, ProtobufCMessage *base)
 	futex_init(&ui->bound);
 	ui->peer = NULL;
 	ui->flags = 0;
-	pr_info(" `- Got %#x peer %#x\n", ui->ue->ino, ui->ue->peer);
-	file_desc_add(&ui->d, ui->ue->id, &unix_desc_ops);
+	pr_info(" `- Got %#x peer %#x (name %s)\n",
+		ui->ue->ino, ui->ue->peer,
+		ui->name ? (ui->name[0] ? ui->name : &ui->name[1]) : "-");
 	list_add_tail(&ui->list, &unix_sockets);
-
-	return 0;
+	return file_desc_add(&ui->d, ui->ue->id, &unix_desc_ops);
 }
 
+struct collect_image_info unix_sk_cinfo = {
+	.fd_type = CR_FD_UNIXSK,
+	.pb_type = PB_UNIX_SK,
+	.priv_size = sizeof(struct unix_sk_info),
+	.collect = collect_one_unixsk,
+	.flags = COLLECT_SHARED,
+};
+
 int collect_unix_sockets(void)
 {
-	int ret;
-
-	pr_info("Reading unix sockets in\n");
-
-	ret = collect_image_sh(CR_FD_UNIXSK, PB_UNIXSK,
-			sizeof(struct unix_sk_info), collect_one_unixsk);
-	if (!ret)
-		ret = read_sk_queues();
-
-	return ret;
+	return read_sk_queues();
 }
 
 int resolve_unix_peers(void)
